"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const utils_1 = require("./utils");
const dep_builder_1 = require("./dep-builder");
const task_manager_1 = require("./task-manager");
/**
 * Set global options for all tasks.
 * @param options
 */
function setGlobalOptions(options) {
    Object.assign(task_manager_1.getGlobalTaskManager().globalOptions, options);
}
exports.setGlobalOptions = setGlobalOptions;
function appendCallback(name, fn) {
    let options = task_manager_1.getGlobalTaskManager().globalOptions;
    let oldFn = options[name];
    options[name] = (...args) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (oldFn) {
            yield oldFn(...args);
        }
        return fn();
    });
}
exports.before = (fn) => appendCallback('before', fn);
exports.after = (fn) => appendCallback('after', fn);
exports.onerror = (fn) => appendCallback('onerror', fn);
var TaskOptions;
(function (TaskOptions) {
    TaskOptions.last = empty();
    function empty() {
        return {
            desc: undefined,
            optionDefs: [],
            strict: task_manager_1.getGlobalTaskManager().globalOptions.strict,
            loading: undefined,
        };
    }
    TaskOptions.empty = empty;
})(TaskOptions || (TaskOptions = {}));
function desc(desc) {
    TaskOptions.last.desc = desc;
}
exports.desc = desc;
function option(rawName, description, config) {
    TaskOptions.last.optionDefs.push([rawName, description, config]);
}
exports.option = option;
function strict() {
    TaskOptions.last.strict = true;
}
exports.strict = strict;
/**
 * Set options for next task.
 * @param options
 */
function setOption(options) {
    Object.assign(TaskOptions.last, options);
}
exports.setOption = setOption;
/**
 * Define a task
 * @param name
 * @param dependencies
 * @param fn
 */
function task(name, dependencies = [], fn) {
    if (utils_1.Is.fn(dependencies)) {
        fn = dependencies;
        dependencies = [];
    }
    const t = {
        name,
        options: {},
        optionDefs: TaskOptions.last.optionDefs,
        desc: TaskOptions.last.desc,
        strict: TaskOptions.last.strict,
        loading: TaskOptions.last.loading,
        rawArgs: [],
        dependencies: dependencies.map(d => {
            if (utils_1.Is.str(d)) {
                return { name: d, options: {} };
            }
            else if (d instanceof dep_builder_1.DepBuilder) {
                return d.toTaskDep();
            }
            return d;
        }),
        fn,
    };
    TaskOptions.last = TaskOptions.empty();
    task_manager_1.getGlobalTaskManager().addTask(t);
    return t;
}
exports.task = task;
//# sourceMappingURL=task.js.map